/**
 * Copyright (C) : INRIA - Domaine de Voluceau, Rocquencourt, B.P. 105, 
 * 78153 Le Chesnay Cedex - France 
 * 
 * contributor(s) : SARDES project - http://sardes.inrialpes.fr
 *
 * Contact : jade@inrialpes.fr
 *
 * This software is a computer program whose purpose is to provide a framework
 * to build autonomic systems, following an architecture-based approach.
 *
 * This software is governed by the CeCILL-C license under French law and 
 * abiding by the rules of distribution of free software. You can use, modify
 * and/or redistribute the software under the terms of the CeCILL-C license as 
 * circulated by CEA, CNRS and INRIA at the following URL 
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a 
 * limited warranty and the software's author, the holder of the economic 
 * rights, and the successive licensors have only limited liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated with 
 * loading,  using,  modifying and/or developing or reproducing the software by 
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate,  and  that  also therefore means  that it is
 * reserved for developers  and  experienced professionals having in-depth 
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling 
 * the security of their systems and/or data to be ensured and,  more generally,
 * to use and operate it in the same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had 
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
package org.objectweb.jasmine.jade.control.repair.actuator;

import java.util.Set;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.julia.control.content.SuperControllerNotifier;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.jasmine.jade.control.repair.api.Actuator;
import org.objectweb.jasmine.jade.control.repair.api.ActuatorException;
import org.objectweb.jasmine.jade.control.repair.api.FailureReport;
import org.objectweb.jasmine.jade.util.FractalUtil;
import org.ow2.jasmine.jade.reflex.api.control.BindingNotificationController;
import org.ow2.jasmine.jade.reflex.util.Reflex;

public class ArchCleaner implements Actuator{

  /* (non-Javadoc)
   * @see org.objectweb.jasmine.jade.control.repair.actuator.lib.Actuator#execute(org.objectweb.jasmine.jade.control.repair.util.report.FailureReport)
   */
  public void execute(FailureReport context) throws ActuatorException {
	  System.out.println("[RepairManager] [ArchCleaner] begin");
    stopImpacted(context.getImpactedComponents_M());
    unbindImpactedToFailed(context.getImpactedComponents_M(), context.getFailedComponents_M());
    removeFailedFromImpacted(context.getFailedComponents_M(),context.getImpactedComponents_M());
    System.out.println("[RepairManager] [ArchCleaner] done");
  }

  /**
   * @param failedComponents_M A set of meta failed components
   */
  private void removeFailedFromImpacted(Set<Component> failedComponents_M, Set<Component> impactedComponents_M){
    for(Component failedCmp_M : failedComponents_M){
      try {
        for(Component sub_M : Fractal.getContentController(failedCmp_M).getFcSubComponents()){
        	
          if(impactedComponents_M.contains(sub_M) && !failedComponents_M.contains(sub_M)){
            // notify sub_M that failedCmp_M is failed
        	  System.out.println("notify " + Fractal.getNameController(sub_M).getFcName()
        			  + " that " + Fractal.getNameController(failedCmp_M).getFcName() + " is failed.");
            Component sub_E = Reflex.getDualComponent(sub_M);
            ((SuperControllerNotifier)Fractal.getSuperController(sub_E)).removedFromFc(Reflex.getDualComponent(failedCmp_M));
          }
        }
      } catch (NoSuchInterfaceException e) {
        // ignored failedCmp_M is primitive
      }
      try {
        for(Component super_M : Fractal.getSuperController(failedCmp_M).getFcSuperComponents()){
          if(impactedComponents_M.contains(super_M) && !failedComponents_M.contains(super_M)){
            // remove failedCmp_M from super_M
        	  System.out.println("remove " + Fractal.getNameController(failedCmp_M).getFcName() 
        			  + " from " + Fractal.getNameController(super_M).getFcName() + ".");
            Component super_E = Reflex.getDualComponent(super_M);
            try {
              Fractal.getContentController(super_E).removeFcSubComponent(Reflex.getDualComponent(failedCmp_M));
            } catch (IllegalContentException e) {
              e.printStackTrace();
            } catch (IllegalLifeCycleException e) {
              e.printStackTrace();
            }
          }
        }
      } catch (NoSuchInterfaceException e) {
        // ignored failedCmp_M has no parent
      }
    }
  }

  /**
   * @param impactedComponents_M
   * @param failedComponents_M
   */
  private void unbindImpactedToFailed(Set<Component> impactedComponents_M, Set<Component> failedComponents_M){
    for(Component impactedCmp_M : impactedComponents_M){
      for(Interface itf_M : FractalUtil.getClientInterfaces(impactedCmp_M)){
        try {
          BindingController bc_M = Fractal.getBindingController(impactedCmp_M);
          Interface servItf_M = (Interface)bc_M.lookupFc(itf_M.getFcItfName());
          if(servItf_M != null){
            Component servItfOwner_M = servItf_M.getFcItfOwner();
            if(failedComponents_M.contains(servItfOwner_M)){
              try {
                ((BindingNotificationController)Reflex.getDualController(impactedCmp_M, "binding-notification-controller")).unbindFcNotification(itf_M.getFcItfName());
              } catch (IllegalBindingException e) {
                e.printStackTrace();
              } catch (IllegalLifeCycleException e) {
                e.printStackTrace();
              }
            }
          }
        } catch (NoSuchInterfaceException e) {
          // ingnored
        }
      }
    }
  }

  /**
   * @param impactedComponents_M
   */
  private void stopImpacted(Set<Component> impactedComponents_M){
    for(Component c_M : impactedComponents_M){
    	 try{
       	    System.out.println("[RepairManager] [ArchCleaner] Stop: " + Fractal.getNameController(c_M).getFcName());
       	  }catch(Exception e){
       		  //ignored
       	  }
      try {
        Reflex.getLifeCycleNotificationController(c_M).stopFcNotification();
      } catch (IllegalLifeCycleException e2) {
        e2.printStackTrace();
      } catch (NoSuchInterfaceException e2) {
        e2.printStackTrace();
      }
      try{
        Component c_E = (Component) Reflex.getDual(c_M);
        Reflex.getLifeCycleNotificationController(c_E).stopFcNotification();
      } catch (NoSuchInterfaceException e1) {
        e1.printStackTrace();
      } catch (IllegalLifeCycleException e) {
        e.printStackTrace();
      } catch(NullPointerException e3){
    	  e3.printStackTrace();
      }
      System.out.println("[RepairManager] [ArchCleaner] Done ");
    }
  }
}
