/**
 * Copyright (C) : INRIA - Domaine de Voluceau, Rocquencourt, B.P. 105, 
 * 78153 Le Chesnay Cedex - France 
 * 
 * contributor(s) : SARDES project - http://sardes.inrialpes.fr
 *
 * Contact : jade@inrialpes.fr
 *
 * This software is a computer program whose purpose is to provide a framework
 * to build autonomic systems, following an architecture-based approach.
 *
 * This software is governed by the CeCILL-C license under French law and 
 * abiding by the rules of distribution of free software. You can use, modify
 * and/or redistribute the software under the terms of the CeCILL-C license as 
 * circulated by CEA, CNRS and INRIA at the following URL 
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a 
 * limited warranty and the software's author, the holder of the economic 
 * rights, and the successive licensors have only limited liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated with 
 * loading,  using,  modifying and/or developing or reproducing the software by 
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate,  and  that  also therefore means  that it is
 * reserved for developers  and  experienced professionals having in-depth 
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling 
 * the security of their systems and/or data to be ensured and,  more generally,
 * to use and operate it in the same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had 
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
package org.objectweb.jasmine.jade.control.repair.actuator;

import java.util.Set;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.jasmine.jade.control.repair.api.ActuatorException;
import org.objectweb.jasmine.jade.control.repair.api.FailureReport;
import org.objectweb.jasmine.jade.util.FractalUtil;
import org.ow2.jasmine.jade.fractal.api.control.ReverseBindingController;
import org.ow2.jasmine.jade.reflex.util.Reflex;

/**
 * BindingReifier
 * 25 juin 07
 * @author <a href="mailto:sylvain.sicard@inrialpes.fr">Sylvain Sicard
 * @author <a href="mailto:julien.legrand@inrialpes.fr">Julien Legrand
 *
 */
public class BindingReifier extends AbstractDelegateActuator {

    // ------------------------------------------------------------------------
    // Implementation of Reifier interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     * 
     * @see fr.jade.service.repair.actuator.reifier.Reifier#propagate(org.objectweb.fractal.api.Component)
     */
  public void execute(FailureReport context) throws ActuatorException {
        delegateActuator.execute(context);
        try {
            for (Component subCmp_M : context.getFailedComponents_M()) {
                recreateBindings(subCmp_M);
            }
        } catch (Exception e) {
            throw new ActuatorException(e);
        }
    }

    // ------------------------------------------------------------------------
    // Private Methods
    // ------------------------------------------------------------------------

    /**
     * update all the binding involved in the repair process for the exec
     * component corresponding to the meta component.
     * 
     * @param cmp_M
     *            the meta component
     * @throws Exception
     */
    protected void recreateBindings(Component cmp_M) throws Exception {
      Set<Interface> clientItfs_M = FractalUtil.getClientInterfaces(cmp_M);
      Set<Interface> serverItfs_M = FractalUtil.getServerInterfaces(cmp_M);
      BindingController bc_M = Fractal.getBindingController(cmp_M);
      System.out.println("restore clients dependencies");
      for(Interface clientItf_M : clientItfs_M){
        Interface serverItf_M = (Interface)bc_M.lookupFc(clientItf_M.getFcItfName());
        if (serverItf_M != null)
          bind(Reflex.getDualInterface(clientItf_M), Reflex.getDualInterface(serverItf_M));
      }
      ReverseBindingController rbc_M = org.ow2.jasmine.jade.fractal.util.FractalUtil.getReverseBindingController(cmp_M);
      System.out.println("restore server dependencies");
      for(Interface serverItf_M : serverItfs_M){
        Object[] sourceItfs_M = rbc_M.lookupBindingSource(serverItf_M.getFcItfName());
        if (sourceItfs_M != null){
          for (Object sourceItf_M : sourceItfs_M){
            Interface sItf_M = (Interface) sourceItf_M;
            if (sItf_M != null)
              bind(Reflex.getDualInterface(sItf_M), Reflex.getDualInterface(serverItf_M));
          }
        }
      }
      System.out.println("ok");
    } 
    
    private void bind(Interface clientIft_E, Interface serverItf_E){
      try {
        System.out.println(Fractal.getNameController((clientIft_E.getFcItfOwner())).getFcName() + 
                           "." + clientIft_E.getFcItfName() + 
                           "->" + Fractal.getNameController((serverItf_E.getFcItfOwner())).getFcName() + 
                           "." + serverItf_E.getFcItfName());
      } catch (NoSuchInterfaceException e1) {
        e1.printStackTrace();
      }
      try {
        Reflex.getBindingNotificationController(clientIft_E.getFcItfOwner()).bindFcNotification(clientIft_E.getFcItfName(), serverItf_E);
      } catch (Exception e) {}
    }

}
