/**
 * Copyright (C) : INRIA - Domaine de Voluceau, Rocquencourt, B.P. 105, 
 * 78153 Le Chesnay Cedex - France 
 * 
 * contributor(s) : SARDES project - http://sardes.inrialpes.fr
 *
 * Contact : jade@inrialpes.fr
 *
 * This software is a computer program whose purpose is to provide a framework
 * to build autonomic systems, following an architecture-based approach.
 *
 * This software is governed by the CeCILL-C license under French law and 
 * abiding by the rules of distribution of free software. You can use, modify
 * and/or redistribute the software under the terms of the CeCILL-C license as 
 * circulated by CEA, CNRS and INRIA at the following URL 
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a 
 * limited warranty and the software's author, the holder of the economic 
 * rights, and the successive licensors have only limited liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated with 
 * loading,  using,  modifying and/or developing or reproducing the software by 
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate,  and  that  also therefore means  that it is
 * reserved for developers  and  experienced professionals having in-depth 
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling 
 * the security of their systems and/or data to be ensured and,  more generally,
 * to use and operate it in the same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had 
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
package org.objectweb.jasmine.jade.control.repair.actuator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.rmi.registry.NamingService;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.jasmine.jade.control.repair.api.ActuatorException;
import org.objectweb.jasmine.jade.control.repair.api.FailureReport;
import org.objectweb.jasmine.jade.util.FractalUtil;
import org.objectweb.jasmine.jade.util.NoSuchComponentException;
import org.ow2.jasmine.jade.reflex.api.control.ReflexController;
import org.ow2.jasmine.jade.reflex.api.factory.GenericFactoryNotification;
import org.ow2.jasmine.jade.reflex.util.Reflex;

/**
 * ComponentReifier
 * 25 juin 07
 * @author <a href="mailto:sylvain.sicard@inrialpes.fr">Sylvain Sicard
 * @author <a href="mailto:julien.legrand@inrialpes.fr">Julien Legrand
 *
 */
public class ComponentReifier extends AbstractDelegateActuator {
    
    /**
     * Comment for <code>ns</code>
     */
    protected NamingService ns;
    protected static final String nsItfName = "registry";
    
    // ------------------------------------------------------------------------
    // Implementation of BindingController interface
    // ------------------------------------------------------------------------

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf)throws NoSuchInterfaceException, IllegalBindingException,IllegalLifeCycleException {
        if (clientItfName.equals(nsItfName))
            ns = (NamingService) serverItf;
        else
            super.bindFc(clientItfName, serverItf);
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() {
        List<String> bindings = new ArrayList<String>(Arrays.asList(super.listFc()));
        bindings.add(nsItfName);
        return bindings.toArray(new String[0]);
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName) throws NoSuchInterfaceException {
        if (clientItfName.equals(nsItfName))
            return ns;
        else
            return super.lookupFc(clientItfName);
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException,IllegalBindingException, IllegalLifeCycleException {
        if (clientItfName.equals(nsItfName))
            ns = null;
        else
            super.unbindFc(clientItfName);
    }
    
    // ------------------------------------------------------------------------
    // Implementation of Reifier interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     * 
     * @see fr.jade.service.repair.actuator.reifier.Reifier#propagate(org.objectweb.fractal.api.Component)
     */
    public void execute(FailureReport context) throws ActuatorException {
        try {      
            System.out.println("[Component reifier] starting ...");
            for (Component cmp_M : context.getFailedComponents_M()) {
                try {
                    ReflexController rc_M = Reflex.getReflexController(cmp_M);
                    GenericFactoryNotification gfNot_E = getGenericFactoryNotification(cmp_M);
                    // get newFcInstance parameters 
                    ComponentType cmpType_E = extractCmpType((ComponentType) cmp_M.getFcType());
                    String cmpCtrlDesc_E = rc_M.getReflexAttribute("controllerDesc");
                    String cmpContent_E = rc_M.getReflexAttribute("contentDesc");
                    // instanciate component
                    Component cmp_E = gfNot_E.newFcInstanceNotification( cmpType_E, cmpCtrlDesc_E, cmpContent_E);
                    // set the name
                    Fractal.getNameController(cmp_E).setFcName(Fractal.getNameController(cmp_M).getFcName());
                    // re-set manually the dual ref
                    rc_M.setCmpRef(cmp_E);
                    Reflex.getReflexController(cmp_E).setCmpRef(cmp_M);
                } catch (NoSuchInterfaceException e) {
                	e.printStackTrace();
                	//throw new ActuatorException("Missing required interface ("+ e.getMessage() + ")");
                } 
            }
        } catch (Exception e) {
        	e.printStackTrace();
        	//throw new ActuatorException(e);
        }

    }

    // ------------------------------------------------------------------------
    // Private Methods
    // ------------------------------------------------------------------------

    /**
     * extract from the meta cmp type the type used to create the component for
     * self-healing
     * 
     * @param t
     * @return
     * @throws Exception
     */
    private ComponentType extractCmpType(ComponentType t) throws Exception {
        String itfName = null;
        InterfaceType[] itRes = null;
        TypeFactory tf = Fractal.getTypeFactory(Fractal.getBootstrapComponent());
        InterfaceType[] it = t.getFcInterfaceTypes();
        int j = 0;
        for (int i = 0; i < it.length; i++) {
            itfName = it[i].getFcItfName();
            if (!((itfName.equals("component")) || (itfName.endsWith("-controller")))) {
                j++;
            }
        }
        itRes = new InterfaceType[j];
        j = 0;
        for (int i = 0; i < it.length; i++) {
            itfName = it[i].getFcItfName();
            if (!((itfName.equals("component")) || (itfName.endsWith("-controller")))) {
                itRes[j] = it[i];
                j++;
            }
        }
        return tf.createFcType(itRes);
    }

    /**
     * @param cmp A meta or exec component reference
     * @return The GenericFactoryNotification interface of the cmp's node. 
     */
    private GenericFactoryNotification getGenericFactoryNotification(Component cmp){
      /*
       try{
        Component mr = FractalUtil.getSuperComponentByPath(cmp, "managed_resources");
        Component node = Fractal.getSuperController(mr).getFcSuperComponents()[0];
        String metaComponentName = Fractal.getNameController(node).getFcName();
        Component factory = ns.lookup(metaComponentName+"_factory");
        return Reflex.getGenericFactoryNotification(factory);
      } catch (NoSuchInterfaceException e) {
        e.printStackTrace();
        return null;
      } catch (NoSuchComponentException e) {
        e.printStackTrace();
        return null;
      }
      */
    	
    	return null;
    }
}
