/**
 * Copyright (C) : INRIA - Domaine de Voluceau, Rocquencourt, B.P. 105, 
 * 78153 Le Chesnay Cedex - France 
 * 
 * contributor(s) : SARDES project - http://sardes.inrialpes.fr
 *
 * Contact : jade@inrialpes.fr
 *
 * This software is a computer program whose purpose is to provide a framework
 * to build autonomic systems, following an architecture-based approach.
 *
 * This software is governed by the CeCILL-C license under French law and 
 * abiding by the rules of distribution of free software. You can use, modify
 * and/or redistribute the software under the terms of the CeCILL-C license as 
 * circulated by CEA, CNRS and INRIA at the following URL 
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a 
 * limited warranty and the software's author, the holder of the economic 
 * rights, and the successive licensors have only limited liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated with 
 * loading,  using,  modifying and/or developing or reproducing the software by 
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate,  and  that  also therefore means  that it is
 * reserved for developers  and  experienced professionals having in-depth 
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling 
 * the security of their systems and/or data to be ensured and,  more generally,
 * to use and operate it in the same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had 
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
package org.objectweb.jasmine.jade.control.repair.actuator;

import java.util.Arrays;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.jasmine.jade.control.repair.api.ActuatorException;
import org.objectweb.jasmine.jade.control.repair.api.FailureReport;
import org.ow2.jasmine.jade.reflex.api.control.ContentNotificationController;
import org.ow2.jasmine.jade.reflex.util.Reflex;


/**
 * ContentReifier
 * 25 juin 07
 * @author <a href="mailto:sylvain.sicard@inrialpes.fr">Sylvain Sicard
 * @author <a href="mailto:julien.legrand@inrialpes.fr">Julien Legrand
 *
 */
public class ContentReifier extends AbstractDelegateActuator {

  /*
   * (non-Javadoc)
   * 
   * @see fr.jade.service.repair.actuator.reifier.Reifier#propagate(org.objectweb.fractal.api.Component)
   */
  public void execute(FailureReport context) throws ActuatorException {
    delegateActuator.execute(context);
    System.out.println("[Content reifier] start reifying");
    try {
      for (Component sub_M : context.getFailedComponents_M()) {
        propagateContent(sub_M);
      }
    } catch (Exception e) {
      throw new ActuatorException(e);
    }
    System.out.println("[Content reifier] finish to reify");
  }

  // ------------------------------------------------------------------------
  // Private Methods
  // ------------------------------------------------------------------------

  /**
   * @param cmp_M
   * @throws Exception
   */
  private void propagateContent(Component cmp_M) throws Exception {
    updateSuperComponent(cmp_M);
    updateSubComponent(cmp_M);
  }

  /**
   * @param cmp_M
   * @throws Exception
   */
  private void updateSuperComponent(Component cmp_M) throws Exception {

    Component repairedCmp_E = Reflex.getDualComponent(cmp_M);
    System.out.println("[Content reifier] updating super component(s) of "
                       + Fractal.getNameController(repairedCmp_E).getFcName());
    Component[] superCmps_M = Fractal.getSuperController(cmp_M).getFcSuperComponents();
    Component superCmp_E = null;
    if (superCmps_M != null) {
      for (Component superCmp_M : superCmps_M) {
        /*
         * super component should have a content controller
         */
        try {
          superCmp_E = Reflex.getReflexController(superCmp_M).getCmpRef();
          ContentNotificationController superCCN_E = Reflex.getContentNotificationController(superCmp_E);
          System.out.println("updating " + Fractal.getNameController(superCmp_E).getFcName() + " ("
                             + Fractal.getNameController(superCmp_M).getFcName() + ")");
          Component subsSuperCmp_E[] = Fractal.getContentController(superCmp_E)
                                              .getFcSubComponents();
          if (!Arrays.asList(subsSuperCmp_E).contains(repairedCmp_E)) {
            superCCN_E.addFcSubComponentNotification(repairedCmp_E);
          }
        } catch (Exception ignored) {
        }
      }
    }
  }

  /**
   * @param cmp_M
   * @throws Exception
   */
  private void updateSubComponent(Component cmp_M) throws Exception {

    Component repairedCmp_E = Reflex.getDualComponent(cmp_M);
    System.out.println("[Content reifier] updating sub component(s) of "
                       + Fractal.getNameController(repairedCmp_E).getFcName());
    try {
      ContentNotificationController ccn_E = Reflex.getContentNotificationController(repairedCmp_E);
      ContentController cc_E = Fractal.getContentController(repairedCmp_E);
      Component[] subCmp_M = Fractal.getContentController(cmp_M).getFcSubComponents();
      for (int i = 0; i < subCmp_M.length; i++) {
        Component subCmp_E = Reflex.getReflexController(subCmp_M[i]).getCmpRef();
        System.out.println("updating " + Fractal.getNameController(subCmp_E).getFcName() + " ("
                           + Fractal.getNameController(subCmp_M[i]).getFcName() + ")");
        if (!Arrays.asList(cc_E.getFcSubComponents()).contains(subCmp_E)) {
          System.out.println("\t\t ADD");
          ccn_E.addFcSubComponentNotification(subCmp_E);
        } else {
          System.out.println("\t\t DON'T ADD");
        }
      }
    } catch (Exception e) {
      System.out.println("Not a composite");
    }
  }

}
