/**
 * Copyright (C) : INRIA - Domaine de Voluceau, Rocquencourt, B.P. 105, 
 * 78153 Le Chesnay Cedex - France 
 * 
 * contributor(s) : SARDES project - http://sardes.inrialpes.fr
 *
 * Contact : jade@inrialpes.fr
 *
 * This software is a computer program whose purpose is to provide a framework
 * to build autonomic systems, following an architecture-based approach.
 *
 * This software is governed by the CeCILL-C license under French law and 
 * abiding by the rules of distribution of free software. You can use, modify
 * and/or redistribute the software under the terms of the CeCILL-C license as 
 * circulated by CEA, CNRS and INRIA at the following URL 
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a 
 * limited warranty and the software's author, the holder of the economic 
 * rights, and the successive licensors have only limited liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated with 
 * loading,  using,  modifying and/or developing or reproducing the software by 
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate,  and  that  also therefore means  that it is
 * reserved for developers  and  experienced professionals having in-depth 
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling 
 * the security of their systems and/or data to be ensured and,  more generally,
 * to use and operate it in the same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had 
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
package org.objectweb.jasmine.jade.control.repair.analyser;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.jasmine.jade.control.repair.api.Event;
import org.objectweb.jasmine.jade.control.repair.api.EventHandler;
import org.objectweb.jasmine.jade.control.repair.api.FailureReportHandler;

/**
 * AbstractFailureAnalyser
 * 20 juin 07
 * @author <a href="mailto:sylvain.sicard@inrialpes.fr">Sylvain Sicard
 *
 */
public abstract class AbstractFailureAnalyser implements EventHandler, BindingController{

  /**
   * Comment for <code>mySelf</code>
   */
  protected Component mySelf;
  protected static final String mySelfItfName = "Component";
  
  /**
   * Comment for <code>frh</code>
   */
  protected FailureReportHandler frh;
  protected static final String frhItfName = "report-out";
  
  /* (non-Javadoc)
   * @see org.objectweb.jasmine.jade.control.repair.util.event.EventHandler#handleEvent(org.objectweb.jasmine.jade.control.repair.util.event.Event)
   */
  public abstract void handleEvent(Event e) throws Exception;
  
  /* (non-Javadoc)
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
   */
  public void bindFc(String clientItfName, Object serverItf) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {
    if (clientItfName.equals(mySelfItfName))
      mySelf = (Component) serverItf;
    else if (clientItfName.equals(frhItfName))
      frh = (FailureReportHandler) serverItf;
    else
      throw new NoSuchInterfaceException(clientItfName);
  }

  /* (non-Javadoc)
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc() {
    return new String[] {frhItfName};
  }

  /* (non-Javadoc)
   * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
   */
  public Object lookupFc(String clientItfName) throws NoSuchInterfaceException {
    if (clientItfName.equals(frhItfName))
      return frh;
    else
      throw new NoSuchInterfaceException(clientItfName);
  }

  /* (non-Javadoc)
   * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
   */
  public void unbindFc(String clientItfName) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException {
    if (clientItfName.equals(frhItfName))
      frh = null;
    else
      throw new NoSuchInterfaceException(clientItfName);
  }

}
