/**
 * Copyright (C) : INRIA - Domaine de Voluceau, Rocquencourt, B.P. 105, 
 * 78153 Le Chesnay Cedex - France 
 * 
 * contributor(s) : SARDES project - http://sardes.inrialpes.fr
 *
 * Contact : jade@inrialpes.fr
 *
 * This software is a computer program whose purpose is to provide a framework
 * to build autonomic systems, following an architecture-based approach.
 *
 * This software is governed by the CeCILL-C license under French law and 
 * abiding by the rules of distribution of free software. You can use, modify
 * and/or redistribute the software under the terms of the CeCILL-C license as 
 * circulated by CEA, CNRS and INRIA at the following URL 
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a 
 * limited warranty and the software's author, the holder of the economic 
 * rights, and the successive licensors have only limited liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated with 
 * loading,  using,  modifying and/or developing or reproducing the software by 
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate,  and  that  also therefore means  that it is
 * reserved for developers  and  experienced professionals having in-depth 
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling 
 * the security of their systems and/or data to be ensured and,  more generally,
 * to use and operate it in the same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had 
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
package org.objectweb.jasmine.jade.control.repair.analyser;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.rmi.registry.NamingService;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.jasmine.jade.control.repair.api.Event;
import org.objectweb.jasmine.jade.control.repair.api.FailureReport;
import org.objectweb.jasmine.jade.control.repair.util.EventBody;
import org.objectweb.jasmine.jade.control.repair.util.EventType;
import org.objectweb.jasmine.jade.control.repair.util.FailureReportImpl;
import org.objectweb.jasmine.jade.osgi.JadeProperties;
import org.objectweb.jasmine.jade.util.FractalUtil;
import org.objectweb.jasmine.jade.util.NoSuchComponentException;

/**
 * NodeFailureAnalyserImpl
 * 20 juin 07
 * @author <a href="mailto:sylvain.sicard@inrialpes.fr">Sylvain Sicard
 *
 */
public class NodeFailureAnalyserImpl extends AbstractFailureAnalyser {

	/**
	 * Comment for <code>ns</code>
	 */
	protected NamingService ns;
	protected static final String nsItfName = "registry";

	
	private Component failedNode_M = null;
	private Component managedResources_M = null;
	
	/* (non-Javadoc)
	 * @see org.objectweb.jasmine.jade.control.repair.analyser.AbstractFailureAnalyser#handleEvent(org.objectweb.jasmine.jade.control.repair.util.event.Event)
	 */
	@Override
	public void handleEvent(Event e) throws Exception {
		if (e.getEventType().equals(EventType.NODEFAILURE)) {
			System.out.println("[RepairManager] [Analyser] Begin");
			EventBody eb = e.getEventBody();
			FailureReport fr = new FailureReportImpl();
			Component sr = ns.lookup(JadeProperties.getInstance()
					.getSystemRepresentationName());
			failedNode_M = FractalUtil.getDirectSubComponentByName(
					sr, (String) eb.get("failedNode"));

			fr.setFailedNode_M(failedNode_M);
			fr.setFailedComponents_M(computeFailedComponents(failedNode_M));
			fr.setImpactedComponents_M(computeImpactedComponents(fr
					.getFailedComponents_M()));

			System.out.println("[RepairManager] [Analyser] Done");

			frh.handleFailureReport(fr);
		}
	}

	/**
	 * @param failedNode_M
	 * @return
	 * @throws NoSuchComponentException
	 * @throws NoSuchInterfaceException
	 */
	private Set<Component> computeFailedComponents(Component failedNode_M) throws NoSuchComponentException, NoSuchInterfaceException{
	System.out.println("[RepairManager] [Analyser] Failed components");
	Set<Component> failedComponents_M = new HashSet<Component>();
     managedResources_M = FractalUtil.getFirstFoundSubComponentByName(failedNode_M,"managed_resources");
//    failedComponents_M.add(failedNode_M);
//    failedComponents_M.add(managedResources_M);
	  for(Component c : Fractal.getContentController(managedResources_M).getFcSubComponents()){
		  try{
		    System.out.println("[RepairManager] [Analyser] \t - " + Fractal.getNameController(c).getFcName());
		  }catch(Exception e){
			  // ignored
		  }
	    failedComponents_M.add(c);
	  }
    System.out.println("[RepairManager] [Analyser] \t - Done");
    return failedComponents_M;
  }

	/**
	 * @param failedComponents_M
	 * @return
	 * @throws NoSuchComponentException
	 * @throws NoSuchInterfaceException
	 */
	private Set<Component> computeImpactedComponents(
			Set<Component> failedComponents_M) throws NoSuchComponentException,
			NoSuchInterfaceException {
		System.out.println("[RepairManager] [Analyser] Impacted components");
		Set<Component> impactedComponents_M = new HashSet<Component>();
		for (Component c_M : failedComponents_M) {
			for (Component c2_M : FractalUtil.getNeighborsComponents(c_M)) {
				
				if (!failedComponents_M.contains(c2_M) &&
					(!c2_M.equals(failedNode_M) && !c2_M.equals(managedResources_M))
				   ) {
					try {
						System.out
								.println("[RepairManager] [Analyser] \t - add "
										+ Fractal.getNameController(c2_M)
												.getFcName());
					} catch (Exception e) {
						//ignored
						e.printStackTrace();
					}
					impactedComponents_M.add(c2_M);
				}else{
					System.out.println("[RepairManager] [Analyser] \t - don't add " + Fractal.getNameController(c2_M).getFcName());
				}
			}
		}
		
		System.out.println("[RepairManager] [Analyser] \t - Done");
		return impactedComponents_M;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.jasmine.jade.control.repair.analyser.AbstractFailureAnalyser#bindFc(java.lang.String, java.lang.Object)
	 */
	@Override
	public void bindFc(String clientItfName, Object serverItf)
			throws NoSuchInterfaceException, IllegalBindingException,
			IllegalLifeCycleException {
		if (clientItfName.equals(nsItfName))
			ns = (NamingService) serverItf;
		else
			super.bindFc(clientItfName, serverItf);
	}

	/* (non-Javadoc)
	 * @see org.objectweb.jasmine.jade.control.repair.analyser.AbstractFailureAnalyser#listFc()
	 */
	@Override
	public String[] listFc() {
		List<String> bindings = new ArrayList<String>(Arrays.asList(super
				.listFc()));
		bindings.add(nsItfName);
		return bindings.toArray(new String[0]);
	}

	/* (non-Javadoc)
	 * @see org.objectweb.jasmine.jade.control.repair.analyser.AbstractFailureAnalyser#lookupFc(java.lang.String)
	 */
	@Override
	public Object lookupFc(String clientItfName)
			throws NoSuchInterfaceException {
		if (clientItfName.equals(nsItfName))
			return ns;
		else
			return super.lookupFc(clientItfName);
	}

	/* (non-Javadoc)
	 * @see org.objectweb.jasmine.jade.control.repair.analyser.AbstractFailureAnalyser#unbindFc(java.lang.String)
	 */
	@Override
	public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
			IllegalBindingException, IllegalLifeCycleException {
		if (clientItfName.equals(nsItfName))
			ns = null;
		else
			super.unbindFc(clientItfName);
	}

}
