/**
 * Copyright (C) : INRIA - Domaine de Voluceau, Rocquencourt, B.P. 105, 
 * 78153 Le Chesnay Cedex - France 
 * 
 * contributor(s) : SARDES project - http://sardes.inrialpes.fr
 *
 * Contact : jade@inrialpes.fr
 *
 * This software is a computer program whose purpose is to provide a framework
 * to build autonomic systems, following an architecture-based approach.
 *
 * This software is governed by the CeCILL-C license under French law and 
 * abiding by the rules of distribution of free software. You can use, modify
 * and/or redistribute the software under the terms of the CeCILL-C license as 
 * circulated by CEA, CNRS and INRIA at the following URL 
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a 
 * limited warranty and the software's author, the holder of the economic 
 * rights, and the successive licensors have only limited liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated with 
 * loading,  using,  modifying and/or developing or reproducing the software by 
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate,  and  that  also therefore means  that it is
 * reserved for developers  and  experienced professionals having in-depth 
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling 
 * the security of their systems and/or data to be ensured and,  more generally,
 * to use and operate it in the same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had 
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
package org.objectweb.jasmine.jade.control.repair.reactor;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.jasmine.jade.control.repair.api.Actuator;
import org.objectweb.jasmine.jade.control.repair.api.FailureReport;
import org.objectweb.jasmine.jade.control.repair.api.FailureReportHandler;

/**
 * BasicReactorImpl
 * 27 juin 07
 * @author <a href="mailto:sylvain.sicard@inrialpes.fr">Sylvain Sicard
 *
 */
public class BasicReactorImpl implements FailureReportHandler, BindingController {
  
  /**
   * Comment for <code>archCleaner</code>
   */
  protected Actuator archCleaner;
  protected static final String archCleanerIftName = "arch-cleaner";
  
  /**
   * Comment for <code>repairPolicy</code>
   */
  protected FailureReportHandler policy;
  protected static final String policyIftName = "repair-policy";
  
  /**
   * Comment for <code>reifiers</code>
   */
  protected Actuator reifier;
  protected static final String reifierItfName = "reifier";
  
  // ------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ------------------------------------------------------------------------

  /*
   * (non-Javadoc)
   * 
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
   *      java.lang.Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException {
    if (clientItfName.equals(archCleanerIftName))
      archCleaner = (Actuator) serverItf;
    else if (clientItfName.equals(policyIftName))
      policy = (FailureReportHandler) serverItf;
    else if (clientItfName.equals("reifier"))
      reifier = (Actuator) serverItf;
    else
      throw new NoSuchInterfaceException(clientItfName);
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc() {
    return new String[] {archCleanerIftName, policyIftName, reifierItfName};
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
   */
  public Object lookupFc(String clientItfName)
      throws NoSuchInterfaceException {
    if (clientItfName.equals(archCleanerIftName))
      return archCleaner;
    else if (clientItfName.equals(policyIftName))
      return policy;
    else if (clientItfName.equals(reifierItfName))
      return reifier;
    else
      throw new NoSuchInterfaceException(clientItfName);
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
   */
  public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
      IllegalBindingException, IllegalLifeCycleException {
    if (clientItfName.equals(archCleanerIftName))
      archCleaner = null;
    else if (clientItfName.equals(policyIftName))
      policy = null;
    else if (clientItfName.equals(reifierItfName))
      reifier = null;
    else
      throw new NoSuchInterfaceException(clientItfName);
  }

  // ------------------------------------------------------------------------
  // Implementation of FailureReportHandler interface
  // ------------------------------------------------------------------------

  /*
   * (non-Javadoc)
   * 
   * @see fr.jade.service.repair.util.event.EventHandler#handleEvent(fr.jade.service.repair.util.event.Event)
   */
  public void handleFailureReport(FailureReport fr) throws Exception {
      if(!fr.getFailedComponents_M().isEmpty()){
    	  
    	System.out.println("[RepairManager] [Reactor] begin");
        // Clean system's architecture
        archCleaner.execute(fr);
        // Apply a repair policy
        policy.handleFailureReport(fr);
        // deploy
        reifier.execute(fr);
        System.out.println("[RepairManager] [Reactor] done ");
      }
  }
}
