/**
 * Copyright (C) : INRIA - Domaine de Voluceau, Rocquencourt, B.P. 105, 
 * 78153 Le Chesnay Cedex - France 
 * 
 * contributor(s) : SARDES project - http://sardes.inrialpes.fr
 *
 * Contact : jade@inrialpes.fr
 *
 * This software is a computer program whose purpose is to provide a framework
 * to build autonomic systems, following an architecture-based approach.
 *
 * This software is governed by the CeCILL-C license under French law and 
 * abiding by the rules of distribution of free software. You can use, modify
 * and/or redistribute the software under the terms of the CeCILL-C license as 
 * circulated by CEA, CNRS and INRIA at the following URL 
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a 
 * limited warranty and the software's author, the holder of the economic 
 * rights, and the successive licensors have only limited liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated with 
 * loading,  using,  modifying and/or developing or reproducing the software by 
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate,  and  that  also therefore means  that it is
 * reserved for developers  and  experienced professionals having in-depth 
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling 
 * the security of their systems and/or data to be ensured and,  more generally,
 * to use and operate it in the same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had 
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
package org.objectweb.jasmine.jade.osgi;

import java.net.Inet4Address;
import java.net.Inet6Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Enumeration;

import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.rmi.registry.Registry;
import org.osgi.framework.BundleContext;

/**
 * @author <a href="mailto:julien.legrand@inrialpes.fr">Julien Legrand
 * 
 */
public class JadeProperties {

	/**
	 * The JadeProperties singleton
	 */
	private static JadeProperties singleton;
	
	/**
	 * 
	 */
	private static InetAddress defaultInetAddress = null;
	
	// ------------------------------------------------------------------------
	// Bundle properties
	// ------------------------------------------------------------------------

	/**
	 * Flag Reflex-mode
	 */
	private static boolean isJadeInReflexMode = true;

	/**
	 * Flag automatic Reflex-mode
	 */
	private static boolean isJadeInAutoReflexMode = true;

	/**
	 * Name of the JadeBoot/Node in Reflex-mode
	 */
	private static String jadeReflexName = null;

	/**
	 * Name of the dual JadeBoot/Node in Reflex-mode
	 */
	private static String jadeReflexDualname = null;

	/**
	 * Fractal RMI registry host
	 */
	private static String jadeRegistryHost = null;

	/**
	 * Fractal RMI registry port
	 */
	private static String jadeRegistryPort;

	/**
	 * Node Discovery service host
	 */
	private static String jadeDiscoveryHost = null;

	/**
	 * Node Discovery service port
	 */
	private static String jadeDiscoveryPort = null;

	/**
	 * JNDI port
	 */
	private static String jadeJndiPort;

	/**
	 * Urls of deployable ADL file
	 */
	private static String jadeUrlsDeployableFile = null;

	/**
	 * Heartbeat pulse period in second
	 */
	private static String jadeHeartbeatPulse = null;

	// ------------------------------------------------------------------------
	// Default Jade properties value
	// ------------------------------------------------------------------------

	/**
	 * Fractal RMI registry port
	 */
	private final static String DEFAULT_JADE_REGISTRY_PORT = new Integer(
			Registry.DEFAULT_PORT).toString();

	/**
	 * Node Discovery service port
	 */
	private final static String DEFAULT_JADE_DISCOVERY_PORT = "9998";

	/**
	 * JNDI port
	 */
	private final static String DEFAULT_JADE_JNDI_PORT = "1239";

	/**
	 * Urls of deployable ADL file
	 */
	private final static String DEFAULT_JADE_URLS_DEPLOYABLE_FILE = "http://proton.inrialpes.fr/~jlegrand/jade/examples/";

	/**
	 * Heartbeat pulse period in second
	 */
	private final static String DEFAULT_JADE_HEARTBEAT_PULSE = "3";

	// ------------------------------------------------------------------------
	// Jade component name
	// ------------------------------------------------------------------------

	/**
	 * System Representation component name
	 */
	private final String systemRepresentationName = "system_representation";

	/**
	 * Name of the composite component containing JadeBoot and JadeNodes
	 */
	private final String jadePlatformName = "jade_platform";

	// ------------------------------------------------------------------------
	// Jade_FDF properties
	// ------------------------------------------------------------------------

	/**
	 * Default value for undefined value in FDF description
	 */
	private final static String FDF_UNDEFINED_VALUE = "UNDEFINED";

	// ------------------------------------------------------------------------
	// Fractal factory properties
	// ------------------------------------------------------------------------

	/**
	 * name of the ADL factory used to boot jade
	 */
	public final static String JADE_BOOT = "org.ow2.jasmine.jade.fractal.adl.JadeFactory";

	/**
	 * name of the ADL factory used to boot jade in Reflex mode
	 */
	public final static String JADE_BOOT_REFLEX = "org.ow2.jasmine.jade.fractal.adl.JadeFactory";

	/**
	 * name of the backend used to create jade component
	 */
	public final static String BOOT_BACKEND = "org.ow2.jasmine.jade.fractal.adl.BootBackend";

	/**
	 * name of the backend used to create jade component in Reflex mode
	 */
	public final static String BOOT_BACKEND_REFLEX = FactoryFactory.FRACTAL_BACKEND;

	/**
	 * name of the Fractal ADL describing the JadeBoot
	 */
	public final static String JADE_BOOT_ADL = "org.objectweb.jasmine.jade.boot.jadeboot.JadeBoot";

	/**
	 * name of the Fractal ADL describing the JadeNode
	 */
	public final static String JADE_NODE_ADL = "org.objectweb.jasmine.jade.boot.jadenode.JadeNode";

	// ------------------------------------------------------------------------
	// Fractal properties
	// ------------------------------------------------------------------------

	/**
	 * Default Jade Julia loader
	 */
	private final static String JULIA_LOADER = "org.objectweb.fractal.julia.loader.DynamicLoader";
	/**
	 * Default Jade Fractal provider (bootstrap-with-name)
	 */
	private final static String FRACTAL_PROVIDER = "org.ow2.jasmine.jade.fractal.julia.JuliaJade";

	/**
	 * Jade Fractal provider in Reflex mode (reflex-bootstrap)
	 */
	private final static String FRACTAL_PROVIDER_REFLEX = "org.ow2.jasmine.jade.reflex.util.ReflexJulia";

	/**
	 * Default Jade Julia configuration files
	 */
	private final static String JULIA_CONFIG = "etc/julia.cfg,etc/julia-fractal-rmi.cfg,etc/julia-jade.cfg,etc/julia-deploy.cfg";

	/**
	 * Jade Julia configuration files for reflex mode
	 */
	private final static String JULIA_CONFIG_REFLEX = "etc/julia.cfg,etc/julia-fractal-rmi.cfg,etc/julia-jade.cfg,etc/julia-deploy.cfg,etc/reflex-julia.cfg,etc/reflex-julia-jade.cfg,etc/reflex-julia-deploy.cfg";

	/**
	 * Default Jade julia.loader.use-context-class-loader property value
	 */
	private final static String JULIA_LOADER_USE_CONTEXT_CLASS_LOADER = "false";

	/**
	 * Default Jade Java security policy file
	 */
	private final static String JAVA_SECURITY_POLICY = "etc/java.policy";
	
	// -------------------------------------------------------------------------
	// static
	// -------------------------------------------------------------------------
	
	/**
	 * @return
	 */
	static {

		InetAddress default_localhost = null;

		/*
		 * ... otherwise, look for an IP address among local network interfaces,
		 * ignoring IPv6 addresses
		 */
		Enumeration<NetworkInterface> interfaces = null;
		try {

			interfaces = NetworkInterface.getNetworkInterfaces();

			while (interfaces.hasMoreElements()
					&& (default_localhost == null || default_localhost
							.isLoopbackAddress())) {

				Enumeration<InetAddress> addresses = interfaces.nextElement()
						.getInetAddresses();

				while (addresses.hasMoreElements()
						&& (default_localhost == null || default_localhost
								.isLoopbackAddress())) {
					InetAddress address = addresses.nextElement();

					/*
					 * keep this address if it is a IPv4 address from the
					 * specified subnetwork if any
					 */
					if (!(address instanceof Inet6Address)) {

						default_localhost = address;
					}
				}
			}

		} catch (SocketException ignored) {
		}

		/*
		 * if no valid address can be found, use a loopback address
		 */
		if (default_localhost == null) {

			try {

				default_localhost = InetAddress.getByName(null);

			} catch (UnknownHostException ignored) {
			}

		}

		defaultInetAddress = default_localhost;

	}

	// ------------------------------------------------------------------------
	// Constructor
	// ------------------------------------------------------------------------

	/**
	 * 
	 */
	public JadeProperties() {
	}

	/**
	 * @param context
	 */
	public JadeProperties(BundleContext context) {
		processBundleContext(context);
	}

	// ------------------------------------------------------------------------
	// Singleton
	// ------------------------------------------------------------------------

	/**
	 * @return The JadeProperties singleton
	 */
	public static synchronized JadeProperties getInstance() {
		if (singleton == null) {
			singleton = new JadeProperties();
		}
		return singleton;
	}

	/**
	 * @return The JadeProperties singleton
	 */
	public static synchronized JadeProperties getInstance(BundleContext context) {
		if (singleton == null) {
			singleton = new JadeProperties(context);
		} else {
			processBundleContext(context);
		}
		return singleton;
	}

	// ------------------------------------------------------------------------
	// Getter / Setter
	// ------------------------------------------------------------------------

	public String getJadeDiscoveryHost() {
		return jadeDiscoveryHost;
	}
	
	public String getJadeDiscoveryPort() {
		return jadeDiscoveryPort;
	}

	public String getJadeHeartbeatPulse() {
		return jadeHeartbeatPulse;
	}

	public String getJadeJndiPort() {
		return jadeJndiPort;
	}

	public boolean isJadeReflex() {
		return isJadeInReflexMode;
	}

	public boolean isJadeReflexAuto() {
		return isJadeInAutoReflexMode;
	}

	public String getJadeReflexDualname() {
		return jadeReflexDualname;
	}

	public String getJadeReflexName() {
		return jadeReflexName;
	}

	public String getJadeRegistryHost() {
		return jadeRegistryHost;
	}

	public String getJadeRegistryPort() {
		return jadeRegistryPort;
	}

	public String getJadeUrlsDeployableFile() {
		return jadeUrlsDeployableFile;
	}
	
	public InetAddress getDefaultInetAddress() {
		return defaultInetAddress;
	}

	public String getSystemRepresentationName() {
		return systemRepresentationName;
	}

	public String getJadePlatformName() {
		return jadePlatformName;
	}

	// ------------------------------------------------------------------------
	// Private Methods
	// ------------------------------------------------------------------------

	private static void processBundleContext(BundleContext context) {

		String property = null;

		/*
		 * julia.loader
		 */
		property = context.getProperty("julia.loader");
		if (property != null)
			System.setProperty("julia.loader", property);
		else
			System.setProperty("julia.loader", JULIA_LOADER);
		/*
		 * java.security.policy
		 */
		property = null;
		property = context.getProperty("java.security.policy");
		if (property != null)
			System.setProperty("java.security.policy", property);
		else
			System.setProperty("java.security.policy", JAVA_SECURITY_POLICY);

		/*
		 * julia.loader.use-context-class-loader
		 */
		property = null;
		property = context.getProperty("julia.loader.use-context-class-loader");
		if (property != null)
			System.setProperty("julia.loader.use-context-class-loader",
					property);
		else
			System.setProperty("julia.loader.use-context-class-loader",
					JULIA_LOADER_USE_CONTEXT_CLASS_LOADER);

		/*
		 * Fractal rmi registry host
		 */
		jadeRegistryHost = context.getProperty("jadeboot.registry.host");
		if (jadeRegistryHost == null
				|| FDF_UNDEFINED_VALUE.equals(jadeRegistryHost)
				|| !isValidHost(jadeRegistryHost)) {

			jadeRegistryHost = defaultInetAddress.getHostAddress();
		}
		/*
		 * Fractal rmi registry port
		 */
		jadeRegistryPort = context.getProperty("jadeboot.registry.port");
		if (jadeRegistryPort == null
				|| FDF_UNDEFINED_VALUE.equals(jadeRegistryPort)) {
			jadeRegistryPort = DEFAULT_JADE_REGISTRY_PORT;
		}

		/*
		 * set the System properties (useful for the FractalBootstrap bundle &
		 * for the Fractal-Reflex service)
		 */
		System.setProperty("registry.host", jadeRegistryHost);
		System.setProperty("registry.port", new Integer(jadeRegistryPort)
				.toString());

		/*
		 * get Discovery service host
		 */
		jadeDiscoveryHost = context.getProperty("jadeboot.discovery.host");
		if (jadeDiscoveryHost == null
				|| FDF_UNDEFINED_VALUE.equals(jadeDiscoveryHost)
				|| !isValidHost(jadeDiscoveryHost)) {
			jadeDiscoveryHost = defaultInetAddress.getHostAddress();
		}

		/*
		 * get Discovery service host
		 */
		jadeDiscoveryPort = context.getProperty("jadeboot.discovery.port");
		if (jadeDiscoveryPort == null
				|| FDF_UNDEFINED_VALUE.equals(jadeDiscoveryPort)) {
			jadeDiscoveryPort = DEFAULT_JADE_DISCOVERY_PORT;
		}

		/*
		 * get the JNDI port
		 */
		jadeJndiPort = context.getProperty("jadeboot.jndi.port");
		if (jadeJndiPort == null || FDF_UNDEFINED_VALUE.equals(jadeJndiPort)) {
			jadeJndiPort = DEFAULT_JADE_JNDI_PORT;
		}

		/*
		 * get heartbeat pulse frenquency in second
		 */
		jadeHeartbeatPulse = context.getProperty("jadenode.heartbeat.pulse");
		if (jadeHeartbeatPulse == null
				|| FDF_UNDEFINED_VALUE.equals(jadeHeartbeatPulse)) {
			jadeHeartbeatPulse = DEFAULT_JADE_HEARTBEAT_PULSE;
		}

		/*
		 * get the URLs of deployable ADL file
		 */
		jadeUrlsDeployableFile = context
				.getProperty("jadeboot.urls.deployable.file");
		if (jadeUrlsDeployableFile == null
				|| FDF_UNDEFINED_VALUE.equals(jadeUrlsDeployableFile)) {

			jadeUrlsDeployableFile = DEFAULT_JADE_URLS_DEPLOYABLE_FILE;
		}

		/*
		 * get Reflex-mode flag
		 */
		property = null;
		property = context.getProperty("jade.reflex");
		if (property != null) {
			isJadeInReflexMode = Boolean.parseBoolean(property);
		}

		if (isJadeInReflexMode) {
			/*
			 * jade.reflex.name
			 */
			property = null;
			property = context.getProperty("jade.reflex.name");
			if (property != null && !FDF_UNDEFINED_VALUE.equals(property)) {
				System.setProperty("reflex-fractal.name", property);
			}
			/*
			 * jade.reflex.dual-name
			 */
			property = null;
			property = context.getProperty("jade.reflex.dual-name");
			if (property != null && !FDF_UNDEFINED_VALUE.equals(property)) {
				System.setProperty("reflex-fractal.dual-name", property);
			}
			/*
			 * jade.reflex.auto
			 */
			property = null;
			property = context.getProperty("jade.reflex.auto");
			if (property == null || FDF_UNDEFINED_VALUE.equals(property)) {
				System.setProperty("reflex-fractal.auto", "true");
			} else {
				System.setProperty("reflex-fractal.auto", property);
			}
		}

		/*
		 * julia.config
		 */
		property = null;
		property = context.getProperty("julia.config");
		if (property != null) {
			System.setProperty("julia.config", property);
		} else {
			if (isJadeInReflexMode) {
				System.setProperty("julia.config", JULIA_CONFIG_REFLEX);
			} else {
				System.setProperty("julia.config", JULIA_CONFIG);
			}
		}

		/*
		 * fractal.provider
		 */
		property = null;
		property = context.getProperty("fractal.provider");
		if (property != null) {
			System.setProperty("fractal.provider", property);
		} else {
			if (isJadeInReflexMode) {
				System.setProperty("fractal.provider", FRACTAL_PROVIDER_REFLEX);
			} else {
				System.setProperty("fractal.provider", FRACTAL_PROVIDER);
			}
		}

	}

	/**
	 * @param host
	 * @return
	 */
	private static boolean isValidHost(String host) {

		if (host != null && host.indexOf('/') == -1) {

			try {

				Inet4Address.getByName(host);
				return true;

			} catch (UnknownHostException ignored) {

				return false;
			}
		}

		return false;
	}
}
