package org.objectweb.jasmine.jade.service.installer;

import java.util.List;

import org.objectweb.fractal.deployment.local.api.Loader;
import org.objectweb.fractal.deployment.local.api.LocallyInstalledPackage;
import org.objectweb.fractal.deployment.local.api.PackageDescription;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleException;
import org.ungoverned.oscar.BundleImpl;

import org.ow2.jasmine.jade.fractal.julia.loader.ClassLoaderWrapper;

public class LocallyInstalledPackageImpl implements LocallyInstalledPackage {

	/**
	 * 
	 */
	private Loader ldr = null;
	/**
	 * 
	 */
	private List<LocallyInstalledPackage> dependencies = null;
	/**
	 * 
	 */
	private PackageDescription pkgDesc = null;
	/**
	 * 
	 */
	private boolean isMarked = false;
	/**
	 * 
	 */
	private Bundle bnd = null;
	/**
	 * If this object was created, then it is needed, thus referenced
	 */
	private int referenceCounter = 1;

	// -------------------------------------------------------------------------
	// Constructor
	// -------------------------------------------------------------------------
	
	public LocallyInstalledPackageImpl(ClassLoaderWrapper wrapper,
			List<LocallyInstalledPackage> dependencies,
			PackageDescription packageDescription) {
		this(wrapper, dependencies, packageDescription, false);
	}

	public LocallyInstalledPackageImpl(Bundle bnd,
			List<LocallyInstalledPackage> dependencies,
			PackageDescription packageDescription, boolean isMarked) {
		this.bnd = bnd;
		ClassLoader cl = ((BundleImpl) bnd).getInfo().getCurrentModule()
				.getClassLoader();
		this.ldr = new ClassLoaderWrapper(cl);
		this.dependencies = dependencies;
		this.pkgDesc = packageDescription;
		this.isMarked = isMarked;
	}

	public LocallyInstalledPackageImpl(Bundle bnd,
			List<LocallyInstalledPackage> dependencies,
			PackageDescription packageDescription) {
		this(bnd, dependencies, packageDescription, false);
	}

	public LocallyInstalledPackageImpl(ClassLoaderWrapper wrapper,
			List<LocallyInstalledPackage> dependencies,
			PackageDescription packageDescription, boolean isMarked) {
		this.ldr = wrapper;
		this.dependencies = dependencies;
		this.pkgDesc = packageDescription;
		this.isMarked = isMarked;
	}
	
	// -------------------------------------------------------------------------
	// Implementation of LocallyInstalledPackage interface
	// -------------------------------------------------------------------------

	public Loader getLoader() {
		return this.ldr;
	}

	public List<LocallyInstalledPackage> getDependencies() {
		return this.dependencies;
	}

	public PackageDescription getPackageDescription() {
		return this.pkgDesc;
	}

	public boolean isMarked() {
		return this.isMarked;
	}

	public void unInstall() {
		if (bnd != null) {
			try {
				bnd.uninstall();
			} catch (BundleException e) {
				e.printStackTrace();
			}
		}
	}

	public int getReferenceCounter() {
		return this.referenceCounter;
	}

	public void increaseReferenceCounter() {
		this.referenceCounter++;
	}

	public void decreaseReferenceCounter() {
		if (this.referenceCounter > 0) {
			this.referenceCounter--;
		}
	}
	
	public Bundle getBundle() {
		return this.bnd;
	}

}
