/**
 * Copyright (C) : INRIA - Domaine de Voluceau, Rocquencourt, B.P. 105, 
 * 78153 Le Chesnay Cedex - France 
 * 
 * contributor(s) : SARDES project - http://sardes.inrialpes.fr
 *
 * Contact : jade@inrialpes.fr
 *
 * This software is a computer program whose purpose is to provide a framework
 * to build autonomic systems, following an architecture-based approach.
 *
 * This software is governed by the CeCILL-C license under French law and 
 * abiding by the rules of distribution of free software. You can use, modify
 * and/or redistribute the software under the terms of the CeCILL-C license as 
 * circulated by CEA, CNRS and INRIA at the following URL 
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a 
 * limited warranty and the software's author, the holder of the economic 
 * rights, and the successive licensors have only limited liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated with 
 * loading,  using,  modifying and/or developing or reproducing the software by 
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate,  and  that  also therefore means  that it is
 * reserved for developers  and  experienced professionals having in-depth 
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling 
 * the security of their systems and/or data to be ensured and,  more generally,
 * to use and operate it in the same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had 
 * knowledge of the CeCILL-C license and that you accept its terms.
 */

package org.objectweb.jasmine.jade.service.installer.osgi;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.deployment.local.api.LocallyInstalledPackage;
import org.objectweb.fractal.deployment.local.api.PackageDescription;
import org.objectweb.jasmine.jade.service.installer.LocallyInstalledPackageImpl;
import org.objectweb.jasmine.jade.util.DebugService;
import org.objectweb.jasmine.jade.util.JadeException;
import org.objectweb.jasmine.jade.util.Logger;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceReference;
import org.ungoverned.oscar.BundleImpl;
import org.ungoverned.oscar.BundleInfo;
import org.ungoverned.osgi.service.bundlerepository.BundleRepositoryServiceExtended;
import org.ungoverned.osgi.service.shell.ShellService;

import org.ow2.jasmine.jade.fractal.api.control.OSGiContextController;
import org.ow2.jasmine.jade.fractal.julia.loader.ClassLoaderWrapper;

/**
 * 
 * @author <a href="mailto:julien.legrand@inrialpes.fr">Julien Legrand
 * @author <a href="mailto:jakub.kornas@inrialpes.fr">Jakub Kornas
 * 
 */
public class OSGiInstallerImpl implements OSGiInstaller, OSGiContextController,
		LifeCycleController {

	/**
	 * 
	 */
	private boolean started = false;

	/**
	 * 
	 */
	private ShellService shell = null;

	/**
	 * 
	 */
	private BundleContext bc;

	/**
	 * 
	 */
	private List<LocallyInstalledPackage> installedPkgs = new ArrayList<LocallyInstalledPackage>();

	/**
	 * 
	 */
	private BundleRepositoryServiceExtended repoService = null;

	// ------------------------------------------------------------------------
	// Constructor
	// ------------------------------------------------------------------------
	public OSGiInstallerImpl() {
	}

	// ------------------------------------------------------------------------
	// Implementation of LifeCycleController interface
	// ------------------------------------------------------------------------

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
	 */
	public String getFcState() {
		if (started)
			return LifeCycleController.STARTED;
		return LifeCycleController.STOPPED;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
	 */
	public void startFc() throws IllegalLifeCycleException {
		if (!started) {
			getShellService();
			started = true;
		} else {
			throw new IllegalLifeCycleException("[OSGI] already started");
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
	 */
	public void stopFc() throws IllegalLifeCycleException {
		if (started) {
			Logger.println(DebugService.info, "[OSGI] oscar stopped");
			started = false;
		} else
			throw new IllegalLifeCycleException("[OSGI] already stopped");
	}

	// ------------------------------------------------------------------------
	// Implementation of Installer interface
	// ------------------------------------------------------------------------

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.objectweb.fractal.deployment.local.api.Installer#install(org.objectweb.fractal.deployment.local.api.PackageDescription)
	 */
	public LocallyInstalledPackage install(PackageDescription packageDescription) {
		return install(packageDescription, false, true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.objectweb.fractal.deployment.local.api.Installer#installAndMark(org.objectweb.fractal.deployment.local.api.PackageDescription)
	 */
	public LocallyInstalledPackage installAndMark(
			PackageDescription packageDescription) {
		return install(packageDescription, true, true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.objectweb.fractal.deployment.local.api.Installer#uninstall(org.objectweb.fractal.deployment.local.api.LocallyInstalledPackage)
	 */
	public void uninstall(LocallyInstalledPackage cmpPkg) throws JadeException {
		cmpPkg.unInstall();
		installedPkgs.remove(cmpPkg);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.objectweb.fractal.deployment.local.api.Installer#getInstalledPackages()
	 */
	public LocallyInstalledPackage[] getInstalledPackages() {
		return installedPkgs.toArray(new LocallyInstalledPackage[] {});
	}

	// ------------------------------------------------------------------------
	// Implementation of OSGiInstaller interface
	// ------------------------------------------------------------------------

	public LocallyInstalledPackage install(
			PackageDescription packageDescription, boolean isStart) {
		return install(packageDescription, false, isStart);
	}

	public LocallyInstalledPackage installAndMark(
			PackageDescription packageDescription, boolean isStart) {
		return install(packageDescription, true, isStart);
	}

	// -------------------------------------------------------------------------
	// Implementation of OSGiContextController interface
	// -------------------------------------------------------------------------

	public void addBundleId(Component component, long bundleId) {

	}

	public BundleContext getBundleContext() {
		return this.bc;
	}

	public long getBundleId(Component component) {
		return 0;
	}

	public void setBundleContext(BundleContext context) {
		this.bc = context;
		ServiceReference obrServiceRef = bc
				.getServiceReference(org.ungoverned.osgi.service.bundlerepository.BundleRepositoryServiceExtended.class
						.getName());
		this.repoService = (BundleRepositoryServiceExtended) bc
				.getService(obrServiceRef);
	}

	// -----------------------------------------------------------------------
	// Private methods
	// -----------------------------------------------------------------------

	/**
	 * @param packageDescription
	 * @param mark
	 * @param isStart
	 * @return
	 */
	private LocallyInstalledPackage install(
			PackageDescription packageDescription, boolean mark, boolean isStart) {
		LocallyInstalledPackage result = null;

		Iterator<LocallyInstalledPackage> iter = installedPkgs.iterator();
		while (iter.hasNext()) {
			result = iter.next();
			if (packageDescription.equals(result.getPackageDescription())) {
				return result;
			}
		}

		/*
		 * Set package properties
		 */
		Map pkgProps = (Map) packageDescription.getPackageProperties();

		if (pkgProps != null && pkgProps.keySet().size() > 0) {

			Iterator it = pkgProps.keySet().iterator();

			while (it.hasNext()) {
				String key = it.next().toString();
				String propValue = null;

				try {
					// TODO: use an OSGi service to set properties, not a shell
					// command
					propValue = (String) pkgProps.get(key);
					shell.executeCommand("sp " + key + " " + propValue,
							System.out, System.err);

				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}

		// Install the bundle defined by packageName (& its dependencies)
		String resourceName = packageDescription.getPackageID();
		List<Bundle> dependencyBundles = null;
		Bundle bnd = null;
		try {

			List<Bundle> dependencyCandidates = this.repoService.deployBundle(
					System.out, System.err, resourceName, isStart);

			if (dependencyCandidates != null && dependencyCandidates.size() > 0) {

				bnd = dependencyCandidates.get(dependencyCandidates.size() - 1);
				dependencyBundles = dependencyCandidates.subList(0,
						dependencyCandidates.size() - 1);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		if (bnd != null) {
			// TODO: this code is oscar-specific. Fix it.
			BundleInfo binf = ((BundleImpl) bnd).getInfo();
			ClassLoader cl = binf.getCurrentModule().getClassLoader();
			List<LocallyInstalledPackage> dependencies = new ArrayList<LocallyInstalledPackage>();

			/*
			 * TODO: This is a very naive and slow implementation. A better data
			 * structure should be used instead.
			 */
			Iterator<Bundle> it = dependencyBundles.iterator();

			while (it.hasNext()) {

				Bundle b = it.next();
				boolean dependencyAlreadyCreated = false;
				Iterator<LocallyInstalledPackage> itr = installedPkgs
						.iterator();

				while (itr.hasNext()) {
					LocallyInstalledPackage dependencyCandidate = itr.next();
					if (b
							.equals(((LocallyInstalledPackageImpl) dependencyCandidate)
									.getBundle())) {
						dependencies.add(dependencyCandidate);
						dependencyCandidate.increaseReferenceCounter();
						dependencyAlreadyCreated = true;
					}
				}

				if (!dependencyAlreadyCreated) {

					/*
					 * FIXME hack to mark jadenode / jadeboot undeployable
					 */
					boolean isJadenodeOrJadeboot = false;

					String bundleName = (String) b.getHeaders().get("Bundle-Name");

					isJadenodeOrJadeboot = "jadenode".equals(bundleName)
							|| "jadeboot".equals(bundleName);

					LocallyInstalledPackage dep = null;
					if (isJadenodeOrJadeboot) {
						dep = new LocallyInstalledPackageImpl(b, null, null,
								true);
					} else {
						dep = new LocallyInstalledPackageImpl(b, null, null,
								false);
					}

					installedPkgs.add(dep);
					dependencies.add(dep);
				}
			}

			result = new LocallyInstalledPackageImpl(bnd, dependencies,
					packageDescription);
		} else {
			// TODO: we're ommitting the OSGi here ... shouldn't be
			result = new LocallyInstalledPackageImpl(new ClassLoaderWrapper(
					this.getClass().getClassLoader()), null, packageDescription);
		}

		installedPkgs.add(result);
		return result;
	}

	/**
	 * Get the shell service from Oscar.
	 */
	private void getShellService() {
		try {
			ServiceReference shellRef[] = bc.getServiceReferences(
					ShellService.class.getName(), null);
			if (shellRef == null) {
				System.out.println("No shell service is available.");
			}
			shell = (ShellService) bc.getService(shellRef[0]);
		} catch (InvalidSyntaxException e) {
			e.printStackTrace();
		}
	}

}
